package system

import (
	"f5"
	"main/constant"
	"main/model/system"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"
)

type AnncApi struct {
}

func (this *AnncApi) AnncList(c *gin.Context) {
	var anncList []system.Annc
	err := f5.GetApp().GetOrmDb(constant.CONF_DB).Find(&anncList).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    anncList,
	})
}

func (this *AnncApi) AddAnnc(c *gin.Context) {
	annc := system.Annc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	var count int64
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(annc.TableName()).Where("model = ?", annc.Model).Where("type = ?", annc.Type).Count(&count)
	if count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "无法再插入同类型的公告",
		})
		return
	}

	err := db.Create(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AnncApi) UpdateAnnc(c *gin.Context) {
	idx, _ := strconv.ParseUint(c.Param("idx"), 10, 64)
	var count int64
	annc := system.Annc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(annc.TableName()).Where("idx = ?", idx).Count(&count)
	if count < 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "不存在的数据",
		})
		return
	}

	db.Table(annc.TableName()).Where("idx != ?", idx).Where("model = ?", annc.Model).Where("type = ?", annc.Type).Count(&count)
	if count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "已有同类型的公告",
		})
		return
	}

	err := db.Select("*").Omit("idx").Where("idx = ?", idx).Updates(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}
