package system

import (
	"f5"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"

	"github.com/gin-gonic/gin"
)

type GameSwitchApi struct {
}

func (this *GameSwitchApi) List(c *gin.Context) {
	pagesize := c.DefaultQuery("pagesize", "10")
	page := c.DefaultQuery("page", "")
	result := []*system.GameSwitch{}
	f5.GetGoStyleDb().PageQuery(
		constant.CONF_DB,
		q5.ToInt32(pagesize),
		q5.ToInt32(page),
		"SELECT * FROM t_game_switch WHERE 1=1",
		[]string{},
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				c.JSON(http.StatusOK, gin.H{
					"code":    1,
					"message": err.Error(),
				})
				return
			}

			for pg.Rows.Next() {
				p := new(system.GameSwitch)
				p.Idx = q5.SafeToInt64(pg.Rows.GetByName("idx"))
				p.Name = pg.Rows.GetByName("switch_name")
				p.Opened = q5.SafeToInt32(pg.Rows.GetByName("is_open"))
				p.Remark = pg.Rows.GetByName("remark")
				p.CreateTime = q5.SafeToInt32(pg.Rows.GetByName("createtime"))
				p.ModifyTime = q5.SafeToInt32(pg.Rows.GetByName("modifytime"))
				q5.AppendSlice(&result, p)
			}
			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (this *GameSwitchApi) Add(c *gin.Context) {
	req := struct {
		Name   string `binding:"required" json:"switch_name"`
		Open   *int32 `binding:"required" json:"is_open"`
		Remark string `binding:"required" json:"remark"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Where("switch_name = ?", req.Name).Take(new(system.GameSwitch)).Error; err == nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "已存在",
		})
		return
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch := new(system.GameSwitch)
	gswitch.Name = req.Name
	gswitch.Opened = *req.Open
	gswitch.Remark = req.Remark
	gswitch.CreateTime = nowDaySeconds
	gswitch.ModifyTime = nowDaySeconds
	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (this *GameSwitchApi) Edit(c *gin.Context) {
	req := struct {
		Name   string `binding:"required" json:"switch_name"`
		Open   *int32 `binding:"required" json:"is_open"`
		Remark string `json:"remark"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	gswitch := new(system.GameSwitch)
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(gswitch, "switch_name = ?", req.Name).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch.Opened = *req.Open
	if req.Remark != "" {
		gswitch.Remark = req.Remark
	}
	gswitch.ModifyTime = nowDaySeconds
	if err := db.Where("switch_name = ?", req.Name).Save(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *GameSwitchApi) Del(c *gin.Context) {
	req := struct {
		Name string `binding:"required" json:"switch_name"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	gswitch := new(system.GameSwitch)
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(gswitch, "switch_name = ?", req.Name).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch.ModifyTime = nowDaySeconds
	if err := db.Where("switch_name = ?", req.Name).Delete(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}
