package system

import (
	"f5"
	"fmt"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"

	"github.com/gin-gonic/gin"
)

type PlayerApi struct {
}

func (pai *PlayerApi) Info(c *gin.Context) {
	type InfoForm struct {
		Playername   string `json:"playername"`
		AccurateName bool   `json:"accurate_name"`
		Account_id   string `json:"account_id"`
		Address      string `json:"address"`
	}

	reqJson := InfoForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))

	filterstr := ""
	if reqJson.Playername != "" {
		if reqJson.AccurateName {
			filterstr = " name = '" + reqJson.Playername + "' "
		} else {
			filterstr = " name like '%" + reqJson.Playername + "%'"
		}

	} else if reqJson.Account_id != "" {
		filterstr = " account_id = '" + reqJson.Account_id + "'"
	} else if reqJson.Address != "" {
		filterstr = " address = '" + reqJson.Address + "'"
	} else {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "input one of playername, account_id, address",
		})
		return
	}
	sql := fmt.Sprintf(`SELECT * FROM t_user WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.Player)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) BagQuery(c *gin.Context) {
	type BagQueryForm struct {
		Account_id string `binding:"required" json:"account_id"`
	}

	reqJson := BagQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	filterstr := " account_id = '" + reqJson.Account_id + "' "
	sql := fmt.Sprintf(`SELECT * FROM t_bag WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.BagItem)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) HeroesQuery(c *gin.Context) {
	type HeroesQueryForm struct {
		Account_id string `binding:"required" json:"account_id"`
	}

	reqJson := HeroesQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	filterstr := " account_id = '" + reqJson.Account_id + "' "
	sql := fmt.Sprintf(`SELECT * FROM t_hero WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.Hero)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) GoldBullionQuery(c *gin.Context) {
	type GoldBullionQueryForm struct {
		Token_id     string `json:"token_id"`
		Open_address string `json:"open_address"`
	}

	reqJson := GoldBullionQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := ""
	if reqJson.Token_id != "" {
		filterstr = " token_id = '" + reqJson.Token_id + "'"
	} else if reqJson.Open_address != "" {
		filterstr = " open_address = '" + reqJson.Open_address + "'"
	} else {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "input one of token_id, open_address",
		})
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_gold_bullion WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.GoldBullion)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) TicketConsumeQuery(c *gin.Context) {
	type TicketConsumeQueryForm struct {
		Room_id    string `binding:"required" json:"room_id"`
		Account_id string `json:"account_id"`
	}

	reqJson := TicketConsumeQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := " match_room_uuid = '" + reqJson.Room_id + "' "
	if reqJson.Account_id != "" {
		filterstr += " AND account_id = '" + reqJson.Account_id + "' "
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_ticket_consume_record WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.TicketConsume)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) GameMallQuery(c *gin.Context) {
	type GameMallQueryForm struct {
		Order_id       string `json:"order_id"`
		Seller         string `json:"seller"`
		Seller_address string `json:"seller_address"`
	}

	reqJson := GameMallQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := ""
	if reqJson.Order_id != "" {
		filterstr = " order_id = '" + reqJson.Order_id + "'"
	} else if reqJson.Seller != "" {
		filterstr = " seller = '" + reqJson.Seller + "'"
	} else if reqJson.Seller_address != "" {
		filterstr = " seller_address = '" + reqJson.Seller_address + "'"
	} else {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "input one of order_id, seller, seller_address",
		})
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_ingame_mall WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.MallItem)
		p.LoadFromDs(ds)
		return p
	})
}
